@@ -5,9 +5,9 @@ import os
 from contextlib import suppress
 from importlib.util import find_spec
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Tuple
 
-from flake8_type_checking.constants import TCH001, TCH002, TCH003, TCH004, TCHA001, TCHA002, TCHB001, TCHB002
+from flake8_type_checking.constants import TC001, TC002, TC003, TC004, TC100, TC101, TC200, TC201
 
 if TYPE_CHECKING:
     from typing import Any, List, Optional
@@ -49,7 +49,7 @@ class ImportVisitor(ast.NodeTransformer):
         self.import_names: dict[str, tuple[str, bool]] = {}
 
         # List of all names and ids, except type declarations - used to find otherwise unused imports
-        self.uses: list[str] = []
+        self.uses: dict[str, ast.AST] = {}
 
         # Tuple of (node, import name) for all import defined within a type-checking block
         self.type_checking_block_imports: set[tuple[ImportType, str]] = set()
@@ -69,14 +69,18 @@ class ImportVisitor(ast.NodeTransformer):
         # Where the type checking block exists (line_start, line_end, col_offset)
         self.type_checking_blocks: List[tuple[int, int, int]] = []
 
+        # Function scopes can tell us if imports that appear in type-checking blocks
+        # are repeated inside a function. This prevents false TC004 positives.
+        self.function_scopes: dict[int, dict] = {}
+
     @property
     def names(self) -> set[str]:
         """Return unique names."""
-        return set(self.uses)
+        return set(self.uses.keys())
 
     # -- Map type checking block ---------------
 
-    def _import_defined_inside_type_checking_block(self, node: ImportType) -> bool:
+    def _in_type_checking_block(self, node: ast.AST) -> bool:
         """Indicate whether an import is defined inside an `if TYPE_CHECKING` block or not."""
         if node.col_offset == 0:
             return False
@@ -138,7 +142,7 @@ class ImportVisitor(ast.NodeTransformer):
 
         :param node: ast.Import or ast.ImportFrom object
         """
-        if self._import_defined_inside_type_checking_block(node):
+        if self._in_type_checking_block(node):
             # For type checking blocks we want to
             # 1. Record annotations for TCH2XX errors
             # 2. Avoid recording imports for TCH1XX errors, by returning early
@@ -149,6 +153,7 @@ class ImportVisitor(ast.NodeTransformer):
                     name = name_node.name
                 self.type_checking_block_imports.add((node, name))
             return None
+
         for name_node in node.names:
             # Check for `from __futures__ import annotations`
             if self.futures_annotation is None:
@@ -173,12 +178,17 @@ class ImportVisitor(ast.NodeTransformer):
                     import_name = module + name_node.name
                 is_local = self._import_is_local(f'{module}{name_node.name}')
                 if is_local:
-                    self.local_imports[import_name] = {'error': TCH001, 'node': node}
+                    self.local_imports[import_name] = {'error': TC001, 'node': node}
                     self.import_names[name] = import_name, True
+
                 else:
-                    self.remote_imports[import_name] = {'error': TCH002, 'node': node}
+                    self.remote_imports[import_name] = {'error': TC002, 'node': node}
                     self.import_names[name] = import_name, False
 
+                if node.lineno not in self.function_scopes:
+                    self.function_scopes[node.lineno] = {'imports': []}
+                self.function_scopes[node.lineno]['imports'].append(name)
+
     def visit_Import(self, node: ast.Import) -> None:
         """Append objects to our import map."""
         self._add_import(node)
@@ -198,13 +208,13 @@ class ImportVisitor(ast.NodeTransformer):
     def visit_Name(self, node: ast.Name) -> ast.Name:
         """Map names."""
         if hasattr(node, 'parent'):
-            self.uses.append(f'{node.id}.{node.parent}')  # type: ignore
-        self.uses.append(node.id)
+            self.uses[f'{node.id}.{node.parent}'] = node  # type: ignore
+        self.uses[node.id] = node
         return node
 
     def visit_Constant(self, node: ast.Constant) -> ast.Constant:
         """Map constants."""
-        self.uses.append(node.value)
+        self.uses[node.value] = node
         return node
 
     # -- Map annotations ------------------------------
@@ -259,15 +269,35 @@ class ImportVisitor(ast.NodeTransformer):
 
     def visit_FunctionDef(self, node: ast.FunctionDef) -> None:
         """Remove and map function arguments and returns."""
+        # Map annotations
         for path in [node.args.args, node.args.kwonlyargs]:
             argument: ast.arg
             for argument in path:
                 if hasattr(argument, 'annotation'):
                     self._add_annotation(argument.annotation)  # type: ignore
                     delattr(argument, 'annotation')
+        if hasattr(node.args, 'kwarg'):
+            kwarg = node.args.kwarg
+            if hasattr(kwarg, 'annotation'):
+                self._add_annotation(kwarg.annotation)  # type: ignore
+                delattr(kwarg, 'annotation')
+        if hasattr(node.args, 'vararg'):
+            vararg = node.args.vararg
+            if hasattr(vararg, 'annotation'):
+                self._add_annotation(vararg.annotation)  # type: ignore
+                delattr(vararg, 'annotation')
         if hasattr(node, 'returns'):
             self._add_annotation(node.returns)  # type: ignore
             delattr(node, 'returns')
+
+        # Register function start and end
+        for i in range(node.lineno, node.end_lineno + 1):  # type: ignore
+            # TODO: Find a better data structure for this
+            if i not in self.function_scopes:
+                self.function_scopes[i] = {'imports': []}
+            self.function_scopes[i]['start'] = node.lineno
+            self.function_scopes[i]['end'] = node.end_lineno + 1  # type: ignore
+
         self.generic_visit(node)
 
 
@@ -287,26 +317,26 @@ class TypingOnlyImportsChecker:
         self.visitor.visit(node)
 
         self.generators = [
-            # TCH001
+            # TC001
             self.unused_import,
-            # TCH002
+            # TC002
             self.unused_third_party_import,
-            # TCH003
+            # TC003
             self.multiple_type_checking_blocks,
-            # TCH004
+            # TC004
             self.used_type_checking_imports,
-            # TCHA001
+            # TC100
             self.missing_futures_import,
-            # TCHA002
+            # TC101
             self.futures_excess_quotes,
-            # TCHB001
+            # TC200
             self.missing_quotes,
-            # TCHB002
+            # TC201
             self.excess_quotes,
         ]
 
     def unused_import(self) -> Flake8Generator:
-        """TCH001."""
+        """TC001."""
         for name in set(self.visitor.import_names) - self.visitor.names:
             unused_import, local_import = self.visitor.import_names[name]
             if local_import:
@@ -316,7 +346,7 @@ class TypingOnlyImportsChecker:
                     yield node.lineno, node.col_offset, error_message.format(module=unused_import), None
 
     def unused_third_party_import(self) -> Flake8Generator:
-        """TCH002."""
+        """TC002."""
         for name in set(self.visitor.import_names) - self.visitor.names:
             unused_import, local_import = self.visitor.import_names[name]
             if not local_import:
@@ -326,30 +356,51 @@ class TypingOnlyImportsChecker:
                     yield node.lineno, node.col_offset, error_message.format(module=unused_import), None
 
     def multiple_type_checking_blocks(self) -> Flake8Generator:
-        """TCH003."""
+        """TC003."""
         if len([i for i in self.visitor.type_checking_blocks if i[2] == 0]) > 1:
-            yield self.visitor.type_checking_blocks[-1][0], 0, TCH003, None
+            yield self.visitor.type_checking_blocks[-1][0], 0, TC003, None
 
     def used_type_checking_imports(self) -> Flake8Generator:
-        """TCH004."""
+        """TC004."""
         for _import, import_name in self.visitor.type_checking_block_imports:
             if import_name in self.visitor.uses:
-                yield _import.lineno, 0, TCH004.format(module=import_name), None
+                # If we get to here, we're pretty sure that the import
+                # shouldn't actually live inside a type-checking block
+
+                use = self.visitor.uses[import_name]
+
+                # before we yield we just want to check one thing: whether the use is inside a type checking block..
+                if self.visitor._in_type_checking_block(use):
+                    return
+
+                # .. or whether there is another duplicate import inside the function scope
+                # (if the use is in a function scope)
+                if use.lineno in self.visitor.function_scopes:
+                    for i in range(
+                        self.visitor.function_scopes[use.lineno]['start'],
+                        self.visitor.function_scopes[use.lineno]['end'],
+                    ):
+                        print(import_name)
+                        print(self.visitor.function_scopes[i]['imports'])
+                        if import_name in self.visitor.function_scopes[i]['imports']:
+                            return
+
+                yield _import.lineno, 0, TC004.format(module=import_name), None
 
     def missing_futures_import(self) -> Flake8Generator:
-        """TCHA001."""
+        """TC100."""
         if (
             not self.visitor.futures_annotation
             and {name for _, name in self.visitor.type_checking_block_imports} - self.visitor.names
         ):
-            yield 1, 0, TCHA001, None
+            yield 1, 0, TC100, None
 
     def futures_excess_quotes(self) -> Flake8Generator:
-        """TCHA002."""
+        """TC101."""
         # If futures imports are present, any ast.Constant captured in _add_annotation should yield an error
         if self.visitor.futures_annotation:
             for (lineno, col_offset, annotation) in self.visitor.wrapped_annotations:
-                yield lineno, col_offset, TCHA002.format(annotation=annotation), None
+                yield lineno, col_offset, TC101.format(annotation=annotation), None
         else:
             """
             If we have no futures import and we have no imports inside a type-checking block, things get more tricky:
@@ -383,17 +434,17 @@ class TypingOnlyImportsChecker:
                         if class_name == annotation:
                             break
                     else:
-                        yield lineno, col_offset, TCHA002.format(annotation=annotation), None
+                        yield lineno, col_offset, TC101.format(annotation=annotation), None
 
     def missing_quotes(self) -> Flake8Generator:
-        """TCHB001."""
+        """TC200."""
         for (lineno, col_offset, annotation) in self.visitor.unwrapped_annotations:
             for _, name in self.visitor.type_checking_block_imports:
                 if annotation == name:
-                    yield lineno, col_offset, TCHB001.format(annotation=annotation), None
+                    yield lineno, col_offset, TC200.format(annotation=annotation), None
 
     def excess_quotes(self) -> Flake8Generator:
-        """TCHB002."""
+        """TC201."""
         for (lineno, col_offset, annotation) in self.visitor.wrapped_annotations:
             # See comment in futures_excess_quotes
             for _, import_name in self.visitor.type_checking_block_imports:
@@ -404,7 +455,7 @@ class TypingOnlyImportsChecker:
                     if class_name == annotation:
                         break
                 else:
-                    yield lineno, col_offset, TCHB002.format(annotation=annotation), None
+                    yield lineno, col_offset, TC201.format(annotation=annotation), None
 
     @property
     def errors(self) -> Flake8Generator:
